# Generated by Django 2.0.2 on 2018-02-09 10:46

from django.db import migrations, models
import django.db.models.deletion
import wxmgmt.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('openid', models.CharField(max_length=100, unique=True)),
                ('nickname', models.CharField(max_length=100)),
                ('sex', models.IntegerField(choices=[(0, '未知'), (1, '男'), (2, '女')])),
                ('city', models.CharField(max_length=50)),
                ('province', models.CharField(max_length=50)),
                ('country', models.CharField(max_length=50)),
                ('headimgurl', models.CharField(max_length=250)),
                ('subscribed_at', models.DateTimeField()),
                ('unionid', models.CharField(max_length=200)),
                ('remark', models.CharField(max_length=100)),
                ('status', models.CharField(default='subscribe', max_length=20)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='MessageTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(default='', max_length=50)),
                ('template_id', models.CharField(max_length=50, unique=True)),
                ('title', models.CharField(blank=True, default='', max_length=50)),
                ('primary_industry', models.CharField(blank=True, default='', max_length=50)),
                ('deputy_industry', models.CharField(blank=True, default='', max_length=50)),
                ('content', models.TextField(blank=True, default='')),
                ('example', models.TextField(blank=True, default='')),
                ('url', models.CharField(blank=True, default='', max_length=500)),
            ],
        ),
        migrations.CreateModel(
            name='Tenant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('chinese_name', models.CharField(help_text='中文名称，标示该商户', max_length=50)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('enable', models.BooleanField(default=True)),
                ('kefuxinxi', models.TextField(blank=True, default='--')),
                ('appid', models.CharField(max_length=200, unique=True)),
                ('secret', models.CharField(help_text='开发者密码', max_length=200, unique=True)),
                ('token', models.CharField(default='', help_text='令牌', max_length=100)),
                ('encoding_aeskey', models.CharField(blank=True, default='', help_text='消息加解密密钥', max_length=200)),
                ('_access_token', models.CharField(blank=True, default='', max_length=200, unique=True)),
                ('access_token_expired_at', models.DateTimeField(blank=True, default=None, null=True)),
                ('_jsapi_ticket', models.CharField(blank=True, default='', max_length=200, unique=True)),
                ('jsapi_ticket_expired_at', models.DateTimeField(blank=True, default=None, null=True)),
                ('shanghu_id', models.CharField(blank=True, default='', help_text='微信支付商户号', max_length=50, unique=True)),
                ('payment_api_key', models.CharField(blank=True, default='', help_text='微信支付商户的API密钥', max_length=200)),
                ('payment_notify_url', models.CharField(blank=True, default='', help_text='支付回调接口', max_length=200)),
                ('refund_notify_url', models.CharField(blank=True, default='', help_text='退款回调接口', max_length=200)),
                ('apiclient_cert_pem', models.FileField(blank=True, default=None, null=True, upload_to=wxmgmt.models.attachment_filename)),
                ('apiclient_key_pem', models.FileField(blank=True, default=None, null=True, upload_to=wxmgmt.models.attachment_filename)),
                ('apiclient_cert_p12', models.FileField(blank=True, default=None, null=True, upload_to=wxmgmt.models.attachment_filename)),
                ('rootca_pem', models.FileField(blank=True, default=None, null=True, upload_to=wxmgmt.models.attachment_filename)),
                ('menu', models.TextField(blank=True, default=None, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('managers', models.ManyToManyField(blank=True, help_text='设置管理员账号，用于接收通知类消息', related_name='parents', to='wxmgmt.Account')),
            ],
        ),
        migrations.AddField(
            model_name='messagetemplate',
            name='tenant',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wxmgmt.Tenant'),
        ),
        migrations.AddField(
            model_name='account',
            name='tenant',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wxmgmt.Tenant'),
        ),
    ]



# Generated by Django 2.0.2 on 2018-02-09 11:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wxmgmt', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='tenant',
            name='kefuxinxi',
        ),
    ]



# Generated by Django 2.0.2 on 2018-02-10 08:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wxmgmt', '0002_remove_tenant_kefuxinxi'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tenant',
            name='_jsapi_ticket',
            field=models.CharField(blank=True, default='', max_length=200),
        ),
        migrations.AlterField(
            model_name='tenant',
            name='shanghu_id',
            field=models.CharField(blank=True, default='', help_text='微信支付商户号', max_length=50),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.shortcuts import render
from django.http import HttpResponse
from django.utils import timezone
from django.views.decorators.http import require_http_methods
from django.views.decorators.csrf import csrf_exempt
from django.conf import settings
import logging, hashlib, json, time, pytz, datetime, uuid
from . import models, signature
logger = logging.getLogger(__name__)

@csrf_exempt
@require_http_methods(['GET','POST'])
def api(request, tenantName):
    """
    微信后端访问入口
    """
    logger.debug(
        'request method(%s),url(%s),params(%s)' % (request.method, request.get_raw_uri(), request.GET))
    signature_params = request.GET.get('signature', '')
    timestamp = request.GET.get('timestamp', '')
    nonce = request.GET.get('nonce', '')
    echostr = request.GET.get('echostr')


    tenant = models.Tenant.objects.filter(name=tenantName).first()
    if not tenant:
        return HttpResponse('没有找到对应的公众号信息--%s' % tenantName)

    # get 请求  配置微信公众号时必须的
    if request.method == 'GET':
        # 安全认证
        hash_sha1 = hashlib.sha1(''.join([tenant.token, timestamp, nonce]).encode("utf8"))
        if hash_sha1.hexdigest() != signature_params:
            logger.debug(u'认证失败, %s, %s' % (hash_sha1, signature_params))
        return HttpResponse(echostr)

    # post请求
    elif request.method == 'POST':
        json.dumps(request.data, indent=4)
        if request.data.get('MsgType') == 'text':
            return render(request, 'text_message.tpl', {
                "ToUserName": request.data.get('FromUserName'),
                "FromUserName": request.data.get('ToUserName'),
                "CreateTime": time.mktime(datetime.datetime.now(pytz.timezone(settings.TIME_ZONE)).timetuple()),
                "Content": u'你好！'
            })

        elif request.data.get('MsgType') == 'event' and request.data.get('Event') == 'subscribe':
            openid = request.data.get('FromUserName')
            account = models.Account.sync_account(tenant, openid)
            return render(request, 'text_message.tpl', {
                "ToUserName": request.data.get('FromUserName'),
                "FromUserName": request.data.get('ToUserName'),
                "CreateTime": time.mktime(datetime.datetime.now(pytz.timezone(settings.TIME_ZONE)).timetuple()),
                "Content": u'%s, 欢迎你：加入合作，门店新增客户、车主免费洗车轻松搞定！' % account.nickname
            })

        elif request.data.get('MsgType') == 'event' and request.data.get('Event') == 'unsubscribe':
            openid = request.data.get('FromUserName')
            account = tenant.account_set.filter(openid=openid).first()
            if account:
                account.status = 'unsubscribe'
                account.save()
            return HttpResponse()

        elif request.data.get('MsgType') == 'event' and request.data.get('Event') == 'CLICK':
            msg = '点击测试'
            if request.data.get('EventKey') == 'kefudianhua':
                msg = tenant.kefuxinxi
            return render(request, 'text_message.tpl', {
                "ToUserName": request.data.get('FromUserName'),
                "FromUserName": request.data.get('ToUserName'),
                "CreateTime": time.mktime(datetime.datetime.now(pytz.timezone(settings.TIME_ZONE)).timetuple()),
                "MsgType": 'text',
                "Content": msg
            })

        elif request.data.get('MsgType') == 'event' and request.data.get('Event') == 'scancode_push':
            return render(request, 'text_message.tpl', {
                "ToUserName": request.data.get('FromUserName'),
                "FromUserName": request.data.get('ToUserName'),
                "CreateTime": time.mktime(datetime.datetime.now(pytz.timezone(settings.TIME_ZONE)).timetuple()),
                "MsgType": 'text',
                "Content": u'扫描结果- '
            })

        elif request.data.get('MsgType') == 'event' and request.data.get('Event') == 'VIEW':
            return render(request, 'text_message.tpl', {
                "ToUserName": request.data.get('FromUserName'),
                "FromUserName": request.data.get('ToUserName'),
                "CreateTime": time.mktime(datetime.datetime.now(pytz.timezone(settings.TIME_ZONE)).timetuple()),
                "MsgType": 'text',
                "Content": u'view跳转 '
            })

        else:
            return HttpResponse()

    # 其他
    else:
        return HttpResponse()


@require_http_methods(['GET'])
def jsticket(request, tenantName):
    """
    #获取 jsapi-ticket
    ###request get
    <pre><code>{
    }</pre></code>

    ###response
    <pre><code>{
        "status": true,
        "message": "success",
        "code": 200,
        "value": {
            "url": "xxx",
            "timestamp": xxxx,
            "signature": "xxxx",
            "appid": "xxxx",
            "noncestr": "xxxx"
        }
    }</pre></code>
    """
    logger.debug('request method(%s),url(%s),body(%s)' % (request.method, request.get_raw_uri(), request.body))
    tenant = models.Tenant.objects.filter(name=tenantName).first()
    if not tenant:
        return HttpResponse('没有找到对应的公众号信息--%s' % tenantName)

    data = {
        "jsapi_ticket": tenant.jsapi_ticket,
        "url": request.META.get('HTTP_REFERER', request.get_raw_uri()),
        "timestamp": int(time.mktime(timezone.now().timetuple())),
        "noncestr": str(uuid.uuid4()).replace('-','')
    }

    data['signature'] = signature(data, 'sha1')
    data['appid'] = tenant.appid
    del data['jsapi_ticket']

    return HttpResponse(data=data)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
from django.db import models
from django.utils import timezone
import logging, requests, json, uuid
from datetime import timedelta, datetime
from . import parsexml, signature

logger = logging.getLogger(__name__)


def attachment_filename(instance, filename):
    file_ext = os.path.splitext(filename)[-1]
    file_name = str(uuid.uuid4()).replace('-', '') + file_ext
    return '%s/%s' % (instance._meta.app_label, file_name)


class Tenant(models.Model):
    chinese_name = models.CharField(max_length=50, help_text=u'中文名称，标示该商户')
    name = models.CharField(max_length=50, unique=True)
    enable = models.BooleanField(default=True)

    # 微信公众号的基础信息
    appid = models.CharField(max_length=200, unique=True)
    secret = models.CharField(max_length=200, unique=True, help_text=u'开发者密码')
    token = models.CharField(max_length=100, help_text=u'令牌', default='')
    encoding_aeskey = models.CharField(blank=True, max_length=200, help_text=u'消息加解密密钥', default='')

    # access_token管理
    _access_token = models.CharField(max_length=200, unique=True, default='', blank=True)
    access_token_expired_at = models.DateTimeField(blank=True, null=True, default=None)

    # jsapi_ticket 管理
    _jsapi_ticket = models.CharField(max_length=200, default='', blank=True)
    jsapi_ticket_expired_at = models.DateTimeField(blank=True, null=True, default=None)

    # 支付相关
    shanghu_id = models.CharField(max_length=50, blank=True, default='', help_text=u'微信支付商户号')
    payment_api_key = models.CharField(max_length=200, blank=True, default='', help_text=u'微信支付商户的API密钥')
    payment_notify_url = models.CharField(max_length=200, blank=True, default='', help_text=u'支付回调接口')
    refund_notify_url = models.CharField(max_length=200, blank=True, default='', help_text=u'退款回调接口')
    apiclient_cert_pem = models.FileField(null=True, blank=True, default=None, upload_to=attachment_filename)
    apiclient_key_pem = models.FileField(null=True, blank=True, default=None, upload_to=attachment_filename)
    apiclient_cert_p12 = models.FileField(null=True, blank=True, default=None, upload_to=attachment_filename)
    rootca_pem = models.FileField(null=True, blank=True, default=None, upload_to=attachment_filename)

    # 微信公众号的菜单按钮
    menu = models.TextField(blank=True, null=True, default=None)

    # 配置管理员
    managers = models.ManyToManyField("Account", blank=True, related_name='parents', help_text=u'设置管理员账号，用于接收通知类消息')

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name

    def __unicode__(self):
        return self.name

    @property
    def access_token(self):
        if self._access_token and self.access_token_expired_at > timezone.now():
            return self._access_token

        url = 'https://api.weixin.qq.com/cgi-bin/token'
        params = {
            'appid': self.appid,
            'secret': self.secret,
            'grant_type': 'client_credential'
        }
        resp = requests.get(url, params=params)
        if resp.status_code != 200:
            logger.error('get access-token failed -- %s %s' % (resp.status_code, resp.content))
            return

        resp_json = resp.json()
        if 'errmsg' in resp_json:
            logger.error('get access-token failed -- %s' % json.dumps(resp_json))
            return

        self._access_token = resp_json['access_token']
        self.access_token_expired_at = timezone.now() + timedelta(seconds=resp_json['expires_in'] - 2)
        self.save()

        return self._access_token

    @property
    def jsapi_ticket(self):
        if self._jsapi_ticket and self.jsapi_ticket_expired_at > timezone.now():
            return self._jsapi_ticket

        if not self.access_token:
            return

        url = 'https://api.weixin.qq.com/cgi-bin/ticket/getticket?type=jsapi'
        params = {
            'access_token': self.access_token
        }
        resp = requests.get(url, params=params)
        if resp.status_code != 200:
            logger.error('get jsapi_ticket failed -- %s %s' % (resp.status_code, resp.content))
            return

        resp_json = resp.json()
        if 'errcode' in resp_json and resp_json['errcode'] != 0:
            logger.error('get jsapi_ticket failed -- %s' % json.dumps(resp_json))
            return

        self._jsapi_ticket = resp_json['ticket']
        self.jsapi_ticket_expired_at = timezone.now() + timedelta(seconds=resp_json['expires_in'] - 2)
        self.save()

        return self._jsapi_ticket


    def get_webauth_openid(self, code):
        url = ' https://api.weixin.qq.com/sns/oauth2/access_token?grant_type=authorization_code'
        params = {
            'code': code,
            'appid': self.appid,
            'secret': self.secret,
        }
        resp = requests.get(url, params=params)
        if resp.status_code != 200:
            logger.error('get openid failed -- %s %s' % (resp.status_code, resp.content))
            return

        resp_json = resp.json()
        if 'errmsg' in resp_json:
            logger.error('get openid failed -- %s' % json.dumps(resp_json))
            return

        return resp_json.get('openid')


    def get_userinfo(self, openid):
        if not self.access_token:
            return

        url = 'https://api.weixin.qq.com/cgi-bin/user/info?lang=zh_CN'
        params = {
            'access_token': self.access_token,
            'openid': openid
        }
        resp = requests.get(url, params=params)
        if resp.status_code != 200:
            logger.error('get userifo failed -- %s %s' % (resp.status_code, resp.content))
            return

        resp_json = resp.json()
        if 'errmsg' in resp_json:
            logger.error('get userifo failed -- %s' % json.dumps(resp_json))
            return

        return resp_json


    def get_openids(self):
        """
        返回一个生成器，每次吐出一个openid
        """
        if not self.access_token:
            return

        url = 'https://api.weixin.qq.com/cgi-bin/user/get'
        params = {
            'access_token': self.access_token,
            'next_openid': ''
        }

        while True:
            logger.debug(u'requests url(%s) params(%s)' % (url, params))
            resp = requests.get(url, params=params)
            if resp.status_code != 200:
                logger.error(u'get openids failed -- %s %s' % (resp.status_code, resp.content))
                return

            resp_json = resp.json()
            if 'errmsg' in resp_json:
                logger.error(u'get openids failed -- %s' % json.dumps(resp_json))
                return

            # 没有关注账号了
            if resp_json.get('count') == 0:
                break

            for i in resp_json['data']['openid']:
                yield i

            # 不需要下一次请求了
            if resp_json.get('count') < 10000:
                break

            params['next_openid'] = resp_json.get('next_openid')


    def get_msg_tpls(self):
        if not self.access_token:
            return

        url = 'https://api.weixin.qq.com/cgi-bin/template/get_all_private_template'
        params = {
            'access_token': self.access_token
        }

        resp = requests.get(url, params=params)
        if resp.status_code != 200:
            logger.error('get MessageTemplates failed -- %s %s' % (resp.status_code, resp.content))
            return

        resp_json = resp.json()
        if 'template_list' not in resp_json:
            logger.error('get MessageTemplates failed -- %s' % json.dumps(resp_json))
            return

        # 设置返回
        return resp_json.get('template_list', [])


    def get_menu(self):
        if not self.access_token:
            return

        url = 'https://api.weixin.qq.com/cgi-bin/menu/get'
        params = {
            'access_token': self.access_token
        }

        resp = requests.get(url, params=params)
        if resp.status_code != 200:
            logger.error('get menu failed -- %s %s' % (resp.status_code, resp.content))
            return

        resp_json = resp.json()
        if not 'menu' in resp_json:
            logger.error('get menu failed -- %s %s' % (resp.status_code, resp.content))
            return

        self.menu = resp.content
        self.save()
        return self


    def delete_menu(self):
        if not self.access_token:
            return

        url = 'https://api.weixin.qq.com/cgi-bin/menu/delete'
        params = {
            'access_token': self.access_token
        }

        resp = requests.get(url, params=params)
        if resp.status_code != 200:
            logger.error('delete menu failed -- %s %s' % (resp.status_code, resp.content))
            return

        resp_json = resp.json()
        if not 'errcode' in resp_json or resp_json.get('errcode') != 0:
            logger.error('delete menu failed -- %s %s' % (resp.status_code, resp.content))
            return

        self.menu = None
        self.save()
        return self


    def create_menu(self):
        if not self.access_token:
            return

        url = 'https://api.weixin.qq.com/cgi-bin/menu/create'
        params = {
            'access_token': self.access_token
        }

        resp = requests.post(url, params=params, data=self.menu.encode('utf-8'))
        if resp.status_code != 200:
            logger.error('create menu failed -- %s %s' % (resp.status_code, resp.content))
            return

        resp_json = resp.json()
        if not 'errcode' in resp_json or resp_json.get('errcode') != 0:
            logger.error('create menu failed -- %s %s' % (resp.status_code, resp.content))
            return
        return self


class Account(models.Model):
    sexes = {
        0: u'未知',
        1: u'男',
        2: u'女'
    }

    tenant = models.ForeignKey(Tenant, on_delete=models.CASCADE)
    openid = models.CharField(max_length=100, unique=True)
    nickname = models.CharField(max_length=100)
    sex = models.IntegerField(choices=sexes.items())
    city = models.CharField(max_length=50)
    province = models.CharField(max_length=50)
    country = models.CharField(max_length=50)
    headimgurl = models.CharField(max_length=250)
    subscribed_at = models.DateTimeField()
    unionid = models.CharField(max_length=200)
    remark = models.CharField(max_length=100)
    status = models.CharField(max_length=20, default='subscribe')

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return '%s-%s' % (self.tenant.name, self.nickname)

    def __unicode__(self):
        return '%s-%s' % (self.tenant.name, self.nickname)

    @classmethod
    def sync_account(cls, tenant, openid):
        userinfo = tenant.get_userinfo(openid)
        if not userinfo:
            return

        if tenant.account_set.filter(openid=openid):
            user = tenant.account_set.get(openid=openid)
            user.nickname = userinfo['nickname']
            user.sex = userinfo['sex']
            user.city = userinfo['city']
            user.province = userinfo['province']
            user.country = userinfo['country']
            user.headimgurl = userinfo['headimgurl']
            user.unionid = userinfo.get('unionid', '')
            user.remark = userinfo['remark']
            user.status = 'subscribe'
            user.save()
        else:
            user = tenant.account_set.create(
                openid=userinfo['openid'],
                nickname=userinfo['nickname'],
                sex=userinfo['sex'],
                city=userinfo['city'],
                province=userinfo['province'],
                country=userinfo['country'],
                headimgurl=userinfo['headimgurl'],
                subscribed_at=datetime.fromtimestamp(userinfo['subscribe_time'], timezone.get_default_timezone()),
                unionid=userinfo.get('unionid', ''),
                remark=userinfo['remark'],
                status='subscribe'
            )
        return user


class MessageTemplate(models.Model):
    tenant = models.ForeignKey(Tenant, on_delete=models.CASCADE)
    key = models.CharField(default='', max_length=50)
    template_id = models.CharField(unique=True, max_length=50)
    title = models.CharField(blank=True, default='', max_length=50)
    primary_industry = models.CharField(blank=True, default='', max_length=50)
    deputy_industry = models.CharField(blank=True, default='', max_length=50)
    content = models.TextField(blank=True, default='')
    example = models.TextField(blank=True, default='')
    url = models.CharField(blank=True, default='', max_length=500)

    def __unicode__(self):
        return self.key

    @staticmethod
    def sync_tpl(tenant, item):
        template_id = item['template_id']
        if tenant.messagetemplate_set.filter(template_id=template_id):
            tpl = tenant.messagetemplate_set.get(template_id=template_id)
            tpl.title = item['title']
            tpl.primary_industry = item['primary_industry']
            tpl.deputy_industry = item['deputy_industry']
            tpl.content = item['content']
            tpl.example = item['example']
            tpl.save()
        else:
            tpl = tenant.messagetemplate_set.create(
                template_id=item['template_id'],
                key=item['template_id'],
                title=item['title'],
                primary_industry=item['primary_industry'],
                deputy_industry=item['deputy_industry'],
                content=item['content'],
                example=item['example']
            )
        return tpl


    def send(self, data, account):
        if not self.access_token:
            return

        url = 'https://api.weixin.qq.com/cgi-bin/message/template/send'
        params = {
            'access_token': self.access_token
        }
        req_data = {
            "touser": account.openid,
            "template_id": self.template_id,
            "url": self.url or None,
            "data": data
        }

        resp = requests.post(url, params=params, data=json.dumps(req_data))
        if resp.status_code != 200:
            logger.error('send TemplateMessage(%s) failed -- %s %s' % (self.key, resp.status_code, resp.content))
            return

        resp_json = resp.json()
        if 'errcode' not in resp_json or resp_json['errcode'] != 0:
            logger.error('send TemplateMessage(%s) failed -- %s' % (self.key, json.dumps(resp_json)))
            return

        logger.debug('send message pass - %s %s ' % (self.key, json.dumps(data)))
        # 设置返回
        return True



# coding = utf8

import logging, hashlib, xml.sax

logger = logging.getLogger(__name__)


def signature(data, _type='sha1', key=None):
    keys = data.keys()
    keys.sort()
    tmpstr = '&'.join('%s=%s' % (k,data[k]) for k in keys)
    if key:
        tmpstr += '&key=' + key
    logger.debug('signature params string %s %s' % (_type, tmpstr))

    tmpstr = tmpstr
    if _type == 'md5':
        hash_md5 = hashlib.md5(tmpstr.encode('utf-8'))
        sign = hash_md5.hexdigest()
    else:
        hash_sha1 = hashlib.sha1(tmpstr)
        sign = hash_sha1.hexdigest()

    logger.debug('signature %s %s' % (_type, sign))
    return sign


def parsexml(xmlstring):
    class XMLHandler(xml.sax.handler.ContentHandler):
        def __init__(self):
            self.buffer = ""
            self.mapping = {}

        def startElement(self, name, attributes):
            self.buffer = ""

        def characters(self, data):
            self.buffer += data

        def endElement(self, name):
            self.mapping[name] = self.buffer

        def getDict(self):
            return self.mapping

    try:
        xmlh = XMLHandler()
        xml.sax.parseString(xmlstring, xmlh)
        return xmlh.getDict()
    except:
        logger.error('parse xmlstring(%s) failed' % xmlstring)
        return


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from .models import Tenant, Account, MessageTemplate
from django.contrib import messages

# Register your models here.
@admin.register(Tenant)
class TenantAdmin(admin.ModelAdmin):
    list_display = [
        'appid', 'name', 'enable', 'access_token_expired_at',
        'jsapi_ticket_expired_at', 'updated_at', 'created_at'
    ]
    filter_horizontal = ['managers']
    list_display_links = ('appid',)
    list_editable = ('name', 'enable')
    actions = ['get_msg_tpls', 'get_menu', 'delete_menu', 'create_menu', 'get_account']

    def get_msg_tpls(self, request, queryset):
        status = 'success'
        for tenant in queryset:
            tpls = tenant.get_msg_tpls()
            if not tpls:
                self.message_user(request, u'同步公众号（%s）的消息模板失败' % tenant.name, level=messages.ERROR)
            for item in tpls:
                MessageTemplate.sync_tpl(tenant, item)
        if status == 'success':
            self.message_user(request, u'同步公众号（%s）的消息模板成功', level=messages.INFO)
    get_msg_tpls.short_description = "获取公众号的消息模板"

    def get_menu(self, request, queryset):
        status = 'success'
        for tenant in queryset:
            if not tenant.get_menu():
                status = 'failed'
                self.message_user(request, u'获取公众号（%s）的menu失败' % tenant.name, level=messages.ERROR)
        if status == 'success':
            self.message_user(request, u'获取公众号的menu成功', level=messages.INFO)
    get_menu.short_description = "获取公众号的自定义菜单"

    def delete_menu(self, request, queryset):
        status = 'success'
        for tenant in queryset:
            if not tenant.delete_menu():
                status = 'failed'
                self.message_user(request, u'删除公众号（%s）的menu失败' % tenant.name, level=messages.ERROR)
        if status == 'success':
            self.message_user(request, u'删除公众号的menu成功', level=messages.INFO)
    delete_menu.short_description = '删除公众号的自定义菜单'

    def create_menu(self, request, queryset):
        status = 'success'
        for tenant in queryset:
            if not tenant.create_menu():
                status = 'failed'
                self.message_user(request, u'创建公众号（%s）的menu失败' % tenant.name, level=messages.ERROR)
        if status == 'success':
            self.message_user(request, u'创建公众号的menu成功', level=messages.INFO)
    create_menu.short_description = '创建公众号的自定义菜单'

    def get_account(self, request, queryset):
        status = 'success'
        for tenant in queryset:
            for openid in tenant.get_openids():
                if not Account.sync_account(tenant, openid):
                    status = 'failed'
                    self.message_user(request, u'同步公众号（%s）的关注账号失败' % tenant.name, level=messages.ERROR)
        if status == 'success':
            self.message_user(request, u'同步公众号的关注账号成功', level=messages.INFO)
    get_account.short_description = '获取公众号的关注用户'



@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = ['nickname', 'tenant', 'openid', 'sex',
        'city', 'province', 'status', 'subscribed_at', 'created_at']
    list_filter = ('tenant', 'status')


@admin.register(MessageTemplate)
class MessageTemplateAdmin(admin.ModelAdmin):
    list_display = ['template_id', 'tenant', 'title', 'key', 'primary_industry', 'deputy_industry']




# coding = utf-8

"""
    @version: v1.0 
    @author: 李大炎 (dayan.li@chinaredstar.com) 
    @contact: 840286247@qq.com 
    @project: PyCharm 
    @file: urls.py 
    @time: 2018/2/25 下午10:55 
"""
from django.urls import path

from . import views

urlpatterns = [
    # ex: /wxmgmt/test/api/
    path('<str:tenantName>/api/', views.api, name='api'),
    path('<str:tenantName>/jsticket/', views.jsticket, name='jsticket')
]